@@ -7,14 +7,14 @@
 #' Type \code{"sparse"} allows a set of efficient algorithms, thus making it more suitable for large sample sizes.
 #' 
 #' An important component of the \code{coco} S4 class is the \code{model.list} specification, involving individual formulas provided as a list, where each of them specifies a covariate-based parametric model for a specific source of nonstationarity. 
-#' It involves \code{"trend"} for the spatial trend, the \code{"std.dev"} for the marginal standard deviation, 
+#' It involves \code{"mean"} for the spatial mean, the \code{"std.dev"} for the marginal standard deviation, 
 #' \code{"scale"}, \code{"aniso"} and \code{"tilt"}, each of them shaping specific aspects of the local spatial geometrically anisotropy structure,
 #' \code{"smooth"} handling local smoothness, and \code{"nugget"} handling the local nugget effect. The models are defined as:
 #' 
 #' 
 #' | Source       | Related to    | Description                             | Model                                                                    |
 #' |--------------|---------------|-----------------------------------------|--------------------------------------------------------------------------|
-#' | \emph{mean}      | \eqn{\mu}       | Mean function                     | \eqn{\boldsymbol{X}_1\boldsymbol{\beta}}                                               |
+#' | \emph{mean}      | \eqn{\mu}       | Spatial mean function             | \eqn{\boldsymbol{X}_1\boldsymbol{\beta}}                                               |
 #' | \emph{std.dev}   | \eqn{\sigma^{X}}| Marginal standard deviation       | \eqn{\text{exp}(0.5 \boldsymbol{X}_2 \boldsymbol{\alpha})}                                    |
 #' | \emph{scale}     | \eqn{\boldsymbol{\Sigma}^{X}}| Local scale          | \eqn{\text{exp}(\boldsymbol{X}_3 \boldsymbol{\theta}_1)}                                      |
 #' | \emph{aniso}     | \eqn{\boldsymbol{\Sigma}^{X}}| Local geometric anisotropy        | \eqn{\text{exp}(\boldsymbol{X}_4 \boldsymbol{\theta}_2)}                         |
@@ -23,7 +23,7 @@
 #' | \emph{nugget}    | \eqn{\sigma^{X}_{\epsilon}}| Local micro-scale variability   | \eqn{\text{exp}(\boldsymbol{X}_7 \boldsymbol{\zeta})}                                |
 #' 
 #' where \eqn{\boldsymbol{\beta}}, \eqn{\boldsymbol{\alpha}}, \eqn{\boldsymbol{\theta}_1}, \eqn{\boldsymbol{\theta}_2}, \eqn{\boldsymbol{\theta}_3}, \eqn{\boldsymbol{\phi}}, and \eqn{\boldsymbol{\zeta}} are the parameter vectors of each model,
-#' \eqn{\nu_{l}}, and \eqn{\nu_{u}} are the lower and upper bounds limiting the range of variation of the spatially-varying smoothness, and where \eqn{\boldsymbol{X}_{\ell}} relates to a specific design matrix defined by the specific models for each of the source of nonstationarity.
+#' \eqn{\nu_{l}}, and \eqn{\nu_{u}} are the lower and upper bounds limiting the range of variation of the spatially-varying smoothness, and where \eqn{\boldsymbol{X}_{\ell}} relates to the design matrix defined by the specific models for each of the source of nonstationarity.
 #' 
 #' Lastly, arguments for the \code{"info"} list argument involve: \itemize{
 #' \item \code{"lambda"}: (\code{numeric}) a positive scalar specifying the regularization parameter. Larger values penalizes highly-smoothed long-tailed covariance functions. 
@@ -38,9 +38,9 @@
 #' @param data (\code{data.frame}) A \code{data.frame} with covariates information, where \code{colnames(data)} matches model.list specification.
 #' @param locs (\code{matrix}) A \code{matrix} with spatial locations. 
 #' @param z (\code{vector} or \code{matrix}) A matrix of \eqn{n \times r} response realizations, one realization per column. When considering only one realization, a vector can also be provided.
-#' @param model.list (\code{list}) A \code{list} specifying a model for each aspect of the spatial structure.
+#' @param model.list (\code{list}) A \code{list} specifying a model for each source of nonstationarity.
 #' @param info (\code{list} or \code{NULL}) A \code{list} specifying characteristics of the coco object.
-#' @param output (\code{list} or \code{NULL}) Empty or the resulting object from running \link[optimParallel]{optimParallel}, adding to this a list with boundary information (check \link{getBoundaries} to check the expected structure).
+#' @param output (\code{list} or \code{NULL}) Empty or the resulting object from running \link[optimParallel]{optimParallel}, adding to this a list with boundary information (see \link{getBoundaries} to check the expected structure).
 #' 
 #' @returns (\code{S4}) An S4 object of class \code{coco}.
 #' 
@@ -88,14 +88,17 @@ coco <- function(type,
   
   .cocons.check.type(type)
   .cocons.check.data(data)
+  
+  if(is.data.frame(locs)){
+    locs <- as.matrix(locs)
+  }
+  
   .cocons.check.locs(locs)
   
   if(is.vector(z)){
     z <- matrix(z, ncol = 1)
   }
   
-
-  
   .cocons.check.z(z, data)
   
   .cocons.check.model.list(model.list,
@@ -123,7 +126,7 @@ coco <- function(type,
     model.list$nugget <- -Inf
   }
   
-  model.list <- model.list[c("mean", "std.dev", "scale", "aniso", "tilt", "smooth", "nugget")]
+  model.list <- model.list[getOption("cocons.Dictionary")]
   
   .cocons.check.info(type = type, 
                     info = info,
