@@ -30,9 +30,9 @@ GetNeg2loglikelihoodTaper <- function(theta, par.pos, ref_taper, locs,
                                                                          rowpointers = ref_taper@rowpointers,
                                                                          smooth_limits =  smooth.limits)
   
-  check_pd <- tryCatch(cholS <-  spam::update.spam.chol.NgPeyton(cholS, ref_taper), error = function(e) e)
+  cholS <- tryCatch(spam::update.spam.chol.NgPeyton(cholS, ref_taper), error = function(e) e)
 
-  if (inherits(check_pd, "error")) {
+  if (inherits(cholS, "error")) {
     if(safe){return(1e+06)} else{
       stop("Cholesky error")
     }
@@ -45,8 +45,7 @@ GetNeg2loglikelihoodTaper <- function(theta, par.pos, ref_taper, locs,
     resid <- x - c(x_covariates %*% theta_list$mean)
     
     n * log(2 * pi) + 2 * c(spam::determinant.spam.chol.NgPeyton(cholS)$modulus) + 
-      sum(resid * spam::solve.spam(cholS, resid))    
-    
+      c(crossprod(spam::forwardsolve.spam(cholS,resid)))
   }))
   
   return(sumlogs + .cocons.getPen(n * dim(z)[2], lambda, theta_list, smooth.limits))
@@ -86,9 +85,9 @@ GetNeg2loglikelihoodTaperProfile <- function(theta, par.pos, ref_taper, locs,
                                                                          rowpointers = ref_taper@rowpointers,
                                                                          smooth_limits =  smooth.limits)
   
-  check_pd <- tryCatch(cholS <-  spam::update.spam.chol.NgPeyton(cholS, ref_taper), error = function(e) e)
+  cholS <- tryCatch(spam::update.spam.chol.NgPeyton(cholS, ref_taper), error = function(e) e)
   
-  if (inherits(check_pd, "error")) {
+  if (inherits(cholS, "error")) {
     if(safe){return(1e+06)} else{
       stop("Cholesky error")
     }
@@ -97,8 +96,7 @@ GetNeg2loglikelihoodTaperProfile <- function(theta, par.pos, ref_taper, locs,
   logdet <- c(spam::determinant.spam.chol.NgPeyton(cholS)$modulus)
   
   sum_in <- sum(apply(z,2,function(x){
-    resid <- x - c(x_covariates %*% theta_list$mean)
-    sum(resid * spam::solve.spam(cholS, resid))
+    c(crossprod(spam::forwardsolve.spam(cholS, x - c(x_covariates %*% theta_list$mean))))
   }))
   
   return(dim(z)[2] * n * log(2 * pi) + 
@@ -121,7 +119,7 @@ GetNeg2loglikelihoodTaperProfile <- function(theta, par.pos, ref_taper, locs,
 #' @param smooth.limits \code{(numeric vector)} smooth.limits.
 #' @param z \code{(numeric vector)} a vector of observed values.
 #' @param n \code{(integer)} dim(z)\[1\].
-#' @param x_betas \code{(matrix) or (data.frame)} design matrix for the trend.
+#' @param x_betas \code{(matrix) or (data.frame)} design matrix for the spatial mean.
 #' @param lambda \code{(numeric)} regularization parameter.
 #' @param safe \code{(TRUE/FALSE)} if \code{TRUE} returns a large pre-defined value under Cholesky errors. Default \code{TRUE}.
 #' @returns value 
@@ -135,9 +133,9 @@ GetNeg2loglikelihoodProfile <- function(theta, par.pos, locs, x_covariates,
                               x_covariates =  x_covariates,
                               smooth_limits = smooth.limits)
   
-  check_pd <- tryCatch(cholS <- base::chol(Sigma_cpp), error = function(e) e)
+  cholS <- tryCatch(base::chol(Sigma_cpp), error = function(e) e)
   
-  if (inherits(check_pd, "error")) {
+  if (inherits(cholS, "error")) {
     if(safe){return(1e+06)} else{
       stop("Cholesky error")
     }
@@ -199,9 +197,9 @@ GetNeg2loglikelihood <- function(theta,
                                x_covariates =  x_covariates,
                                smooth_limits = smooth.limits)
   
-  check_pd <- tryCatch(cholS <- base::chol(Sigma_cpp), error = function(e) e)
+  cholS <- tryCatch(base::chol(Sigma_cpp), error = function(e) e)
   
-  if (inherits(check_pd, "error")) {
+  if (inherits(cholS, "error")) {
     if(safe){return(1e+06)} else{
       stop("Cholesky error")
     }
@@ -213,14 +211,84 @@ GetNeg2loglikelihood <- function(theta,
   
   sum_logliks <- sum(apply(z, 2, function(x){
     tmp_a <- x - tmp_trend
-    n * log(2 * pi) + 2 * logdet + sum(tmp_a * backsolve(cholS,
-                                                         forwardsolve(cholS, 
-                                                                      tmp_a, 
-                                                                      transpose = TRUE, 
-                                                                      upper.tri = TRUE),
-                                                         n))
+    n * log(2 * pi) + 2 * logdet + c(crossprod(forwardsolve(cholS, 
+                                                        tmp_a, 
+                                                        transpose = TRUE, 
+                                                        upper.tri = TRUE)))
   }))
   
   return(sum_logliks +  .cocons.getPen(n * dim(z)[2], lambda, theta_list, smooth.limits))
   
 }
+
+#' GetNeg2loglikelihoodREML
+#' @description compute the negative 2 log REML likelihood based on theta
+#'
+#' @usage GetNeg2loglikelihoodREML(theta, par.pos, locs, x_covariates, x_betas,
+#' smooth.limits, z, n, lambda, safe = TRUE)
+#' @param theta \code{(numeric vector)} a vector with parameters values.
+#' @param par.pos \code{(list)} par.pos list.
+#' @param locs \code{(matrix)} spatial location matrix.
+#' @param x_covariates \code{(data.frame)} design matrix.
+#' @param x_betas \code{(matrix) or (data.frame)} design matrix for the spatial mean.
+#' @param smooth.limits \code{(numeric vector)} smooth.limits.
+#' @param z \code{(numeric vector)} a vector of contrasts.
+#' @param n \code{(integer)} dim(z)\[1\].
+#' @param lambda \code{(numeric)} regularization parameter.
+#' @param safe \code{(TRUE/FALSE)} if \code{TRUE} returns a large pre-defined value under Cholesky errors. Default \code{TRUE}.
+#' @returns value
+#' @author Federico Blasi
+GetNeg2loglikelihoodREML <- function(theta, 
+                                      par.pos, 
+                                      locs, 
+                                      x_covariates, 
+                                      x_betas,
+                                      smooth.limits, 
+                                      z, 
+                                      n,
+                                      lambda,
+                                      safe = TRUE) {
+  
+  theta_list <- cocons::getModelLists(theta = theta, par.pos = par.pos, type = "diff")
+  
+  Sigma_cpp <- cocons::cov_rns(theta = theta_list[-1], 
+                               locs = locs, 
+                               x_covariates =  x_covariates,
+                               smooth_limits = smooth.limits)
+  
+  cholS <- tryCatch(base::chol(Sigma_cpp), error = function(e) e)
+  
+  if (inherits(cholS, "error")) {
+    if(safe){return(1e+06)} else{
+      stop("Cholesky error")
+    }
+  }
+  
+  logdet <- sum(log(diag(cholS)))
+  sum_logliks <- 0
+
+  p <- qr(x_covariates)$rank
+  
+  if(T){
+    V <- backsolve(cholS, forwardsolve(cholS, x_covariates,
+                                       transpose = TRUE, 
+                                       upper.tri = TRUE))
+    W <- crossprod(x_covariates, V)
+    Sigma_inv <- chol2inv(cholS)
+    P_mat <- Sigma_inv - V %*% solve(W, t(V))
+    
+    chol_W <- chol(W)
+  }
+  
+  sum_logliks <- sum(apply(z, 2, function(x){
+    
+    (n - p) * log(2 * pi) + 2 * logdet + 2 * sum(log(diag(chol_W))) +  c(crossprod(x, P_mat %*% x))
+    
+  }))
+  
+  return(sum_logliks +  .cocons.getPen((n - p) * dim(z)[2], lambda, theta_list, smooth.limits))
+  
+}
+
+
+
